@@ -286,6 +286,11 @@ static const int protocol_priority[] = {
   0
 };
 
+static const int dtls_protocol_priority[] = {
+  GNUTLS_DTLS1_0,
+  0
+};
+
 static const int protocol_priority_suiteb[] = {
   GNUTLS_TLS1_2,
   0
@@ -293,10 +298,26 @@ static const int protocol_priority_suiteb[] = {
 
 static const int kx_priority_performance[] = {
   GNUTLS_KX_RSA,
+#ifdef ENABLE_ECDHE
   GNUTLS_KX_ECDHE_ECDSA,
   GNUTLS_KX_ECDHE_RSA,
+#endif
+#ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
+#endif
+  0
+};
+
+static const int kx_priority_pfs[] = {
+#ifdef ENABLE_ECDHE
+  GNUTLS_KX_ECDHE_ECDSA,
+  GNUTLS_KX_ECDHE_RSA,
+#endif
+#ifdef ENABLE_DHE
+  GNUTLS_KX_DHE_RSA,
+  GNUTLS_KX_DHE_DSS,
+#endif
   0
 };
 
@@ -307,11 +328,17 @@ static const int kx_priority_suiteb[] = {
 
 static const int kx_priority_export[] = {
   GNUTLS_KX_RSA,
+#ifdef ENABLE_ECDHE
   GNUTLS_KX_ECDHE_ECDSA,
   GNUTLS_KX_ECDHE_RSA,
+#endif
+#ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
+#endif
+#ifdef ENABLE_RSA_EXPORT
   GNUTLS_KX_RSA_EXPORT,
+#endif
   0
 };
 
@@ -319,64 +346,41 @@ static const int kx_priority_secure[] = {
   /* The ciphersuites that offer forward secrecy take
    * precedence
    */
+#ifdef ENABLE_ECDHE
   GNUTLS_KX_ECDHE_ECDSA,
   GNUTLS_KX_ECDHE_RSA,
+#endif
   GNUTLS_KX_RSA,
   /* KX-RSA is now ahead of DHE-RSA and DHE-DSS due to the compatibility
    * issues the DHE ciphersuites have. That is, one cannot enforce a specific
    * security level without dropping the connection. 
    */
+#ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
+#endif
   /* GNUTLS_KX_ANON_DH: Man-in-the-middle prone, don't add!
    * GNUTLS_KX_RSA_EXPORT: Deprecated, don't add!
    */
   0
 };
 
-static const int cipher_priority_performance_sw[] = {
+static const int cipher_priority_performance[] = {
   GNUTLS_CIPHER_ARCFOUR_128,
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
   GNUTLS_CIPHER_3DES_CBC,
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
-/* If GCM and AES acceleration is available then prefer
- * them over anything else.
- */
-static const int cipher_priority_performance_hw_aes[] = {
+static const int cipher_priority_normal[] = {
   GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_AES_256_GCM,
-  GNUTLS_CIPHER_AES_256_CBC,
-  GNUTLS_CIPHER_ARCFOUR_128,
-  GNUTLS_CIPHER_CAMELLIA_128_CBC,
-  GNUTLS_CIPHER_CAMELLIA_256_CBC,
-  GNUTLS_CIPHER_3DES_CBC,
-  0
-};
-
-static const int cipher_priority_normal_sw[] = {
   GNUTLS_CIPHER_AES_128_CBC,
-  GNUTLS_CIPHER_CAMELLIA_128_CBC,
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_CBC,
-  GNUTLS_CIPHER_CAMELLIA_256_CBC,
-  GNUTLS_CIPHER_AES_256_GCM,
-  GNUTLS_CIPHER_3DES_CBC,
-  GNUTLS_CIPHER_ARCFOUR_128,
-  0
-};
-
-static const int cipher_priority_normal_hw_aes[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_128_CBC,
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
@@ -385,10 +389,6 @@ static const int cipher_priority_normal_hw_aes[] = {
   0
 };
 
-static const int *cipher_priority_performance = cipher_priority_performance_sw;
-static const int *cipher_priority_normal = cipher_priority_normal_sw;
-
-
 static const int cipher_priority_suiteb128[] = {
   GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_256_GCM,
@@ -402,30 +402,32 @@ static const int cipher_priority_suiteb192[] = {
 
 
 static const int cipher_priority_secure128[] = {
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_256_GCM,
+
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
-  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
-  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
 
 static const int cipher_priority_secure192[] = {
+  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
-  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
 /* The same as cipher_priority_security_normal + arcfour-40. */
 static const int cipher_priority_export[] = {
+  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
-  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_3DES_CBC,
   GNUTLS_CIPHER_ARCFOUR_128,
   GNUTLS_CIPHER_ARCFOUR_40,
@@ -539,21 +541,18 @@ typedef void (rmadd_func) (priority_st * priority_list, unsigned int alg);
 static void
 prio_remove (priority_st * priority_list, unsigned int algo)
 {
-  int i = 0;
-  int pos = -1;                 /* the position of the cipher to remove */
+  unsigned int i;
 
-  while (priority_list->priority[i] != 0)
+  for (i=0;i<priority_list->algorithms;i++)
     {
-      if (priority_list->priority[i] == algo)
-        pos = i;
-      i++;
-    }
-
-  if (pos >= 0)
-    {
-      priority_list->priority[pos] = priority_list->priority[i - 1];
-      priority_list->priority[i - 1] = 0;
-      priority_list->algorithms--;
+      if (priority_list->priority[i] == algo) 
+        {
+          priority_list->algorithms--;
+          if ((priority_list->algorithms-i) > 0)
+            memmove(&priority_list->priority[i], &priority_list->priority[i+1], (priority_list->algorithms-i)*sizeof(priority_list->priority[0]));
+          priority_list->priority[priority_list->algorithms] = 0;
+          break;
+        }
     }
 
   return;
@@ -562,19 +561,19 @@ prio_remove (priority_st * priority_list, unsigned int algo)
 static void
 prio_add (priority_st * priority_list, unsigned int algo)
 {
-  register int i = 0;
-  while (priority_list->priority[i] != 0)
+  unsigned int i, l = priority_list->algorithms;
+
+  if (l >= MAX_ALGOS)
+    return;                     /* can't add it anyway */
+
+  for (i = 0; i < l; ++i)
     {
       if (algo == priority_list->priority[i])
         return;                 /* if it exists */
-      i++;
     }
 
-  if (i < MAX_ALGOS)
-    {
-      priority_list->priority[i] = algo;
-      priority_list->algorithms++;
-    }
+  priority_list->priority[l] = algo;
+  priority_list->algorithms++;
 
   return;
 }
@@ -625,6 +624,7 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
 
 #define LEVEL_NONE "NONE"
 #define LEVEL_NORMAL "NORMAL"
+#define LEVEL_PFS "PFS"
 #define LEVEL_PERFORMANCE "PERFORMANCE"
 #define LEVEL_SECURE128 "SECURE128"
 #define LEVEL_SECURE192 "SECURE192"
@@ -664,6 +664,19 @@ bulk_rmadd_func *func;
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
 
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
+      return 1;
+    }
+  else if (strcasecmp (level, LEVEL_PFS) == 0)
+    {
+      func (&priority_cache->cipher, cipher_priority_normal);
+      func (&priority_cache->kx, kx_priority_pfs);
+      func (&priority_cache->mac, mac_priority_normal);
+      func (&priority_cache->sign_algo,
+                     sign_priority_default);
+      func (&priority_cache->supported_ecc, supported_ecc_normal);
+
       if (priority_cache->level == 0)
         priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       return 1;
@@ -768,6 +781,10 @@ bulk_rmadd_func *func;
  * included as a fallback only.  The ciphers are sorted by security
  * margin.
  *
+ * "PFS" means all "secure" ciphersuites that support perfect forward secrecy. 
+ * The 256-bit ciphers are included as a fallback only.  
+ * The ciphers are sorted by security margin.
+ *
  * "SECURE128" means all "secure" ciphersuites of security level 128-bit
  * or more.
  *
@@ -864,7 +881,6 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
       _set_priority (&(*priority_cache)->cert_type, cert_type_priority_default);
       _set_priority (&(*priority_cache)->sign_algo, sign_priority_default);
       _set_priority (&(*priority_cache)->supported_ecc, supported_ecc_normal);
-      (*priority_cache)->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       i = 0;
     }
   else
@@ -912,6 +928,11 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
                   bulk_fn (&(*priority_cache)->protocol,
                                  protocol_priority);
                 }
+              else if (strncasecmp (&broken_list[i][1], "VERS-DTLS-ALL", 13) == 0)
+                {
+                  bulk_fn (&(*priority_cache)->protocol,
+                                 dtls_protocol_priority);
+                }
               else
                 {
                   if ((algo =
@@ -1238,15 +1259,6 @@ gnutls_set_default_export_priority (gnutls_session_t session)
   return gnutls_priority_set_direct (session, "EXPORT", NULL);
 }
 
-/* Increases the priority of AES-GCM as it is much faster
- * than anything else if hardware support is there.
- */
-void _gnutls_priority_prefer_aes_gcm(void)
-{
-  cipher_priority_performance = cipher_priority_performance_hw_aes;
-  cipher_priority_normal = cipher_priority_normal_hw_aes;
-}
-
 /**
  * gnutls_priority_ecc_curve_list:
  * @pcache: is a #gnutls_prioritity_t structure.
