@@ -57,6 +57,7 @@ typedef struct _cfg_ctx
   char *unit;
   char *locality;
   char *state;
+  char *dn;
   char *cn;
   char *uid;
   char *challenge_password;
@@ -213,6 +214,10 @@ template_parse (const char *template)
   if (val != NULL && val->valType == OPARG_TYPE_STRING)
     cfg.state = strdup(val->v.strVal);
 
+  val = optionGetValue(pov, "dn");
+  if (val != NULL && val->valType == OPARG_TYPE_STRING)
+    cfg.dn = strdup(val->v.strVal);
+
   val = optionGetValue(pov, "cn");
   if (val != NULL && val->valType == OPARG_TYPE_STRING)
     cfg.cn = strdup(val->v.strVal);
@@ -283,6 +288,7 @@ template_parse (const char *template)
   READ_NUMERIC("expiration_days", cfg.expiration_days);
   READ_NUMERIC("crl_next_update", cfg.crl_next_update);
   READ_NUMERIC("crl_number", cfg.crl_number);
+  READ_NUMERIC("path_len", cfg.path_len);
   
   val = optionGetValue(pov, "proxy_policy_language");
   if (val != NULL && val->valType == OPARG_TYPE_STRING)
@@ -293,7 +299,6 @@ template_parse (const char *template)
   
   READ_BOOLEAN("ca", cfg.ca);
   READ_BOOLEAN("honor_crq_extensions", cfg.crq_extensions);
-  READ_BOOLEAN("path_len", cfg.path_len);
   READ_BOOLEAN("tls_www_client", cfg.tls_www_client);
   READ_BOOLEAN("tls_www_server", cfg.tls_www_server);
   READ_BOOLEAN("signing_key", cfg.signing_key);
@@ -422,24 +427,29 @@ read_str (const char *input_str)
   return input;
 }
 
-/* Default is no
+/* Default is:
+ * def: 0 -> no
+ * def: 1 -> yes
  */
 int
-read_yesno (const char *input_str)
+read_yesno (const char *input_str, int def)
 {
   char input[128];
 
+restart:
   fputs (input_str, stderr);
   if (fgets (input, sizeof (input), stdin) == NULL)
-    return 0;
+    return def;
 
   if (IS_NEWLINE(input))
-    return 0;
+    return def;
 
   if (input[0] == 'y' || input[0] == 'Y')
     return 1;
-
-  return 0;
+  else if (input[0] == 'n' || input[0] == 'N')
+    return 0;
+  else
+    goto restart;
 }
 
 
@@ -663,6 +673,26 @@ get_cn_crt_set (gnutls_x509_crt_t crt)
 
 }
 
+void
+get_dn_crt_set (gnutls_x509_crt_t crt)
+{
+  int ret;
+  const char* err;
+
+  if (batch)
+    {
+      if (!cfg.dn)
+        return;
+      ret =
+        gnutls_x509_crt_set_dn (crt, cfg.dn, &err);
+      if (ret < 0)
+        {
+          fprintf (stderr, "set_dn: %s at: %s\n", gnutls_strerror (ret), err);
+          exit (1);
+        }
+    }
+}
+
 void
 get_uid_crt_set (gnutls_x509_crt_t crt)
 {
@@ -718,7 +748,7 @@ get_oid_crt_set (gnutls_x509_crt_t crt)
 }
 
 void
-get_key_purpose_set (gnutls_x509_crt_t crt)
+get_key_purpose_set (int type, void *crt)
 {
   int ret, i;
 
@@ -728,9 +758,12 @@ get_key_purpose_set (gnutls_x509_crt_t crt)
         return;
       for (i = 0; cfg.key_purpose_oids[i] != NULL; i++)
         {
-          ret =
-            gnutls_x509_crt_set_key_purpose_oid (crt, cfg.key_purpose_oids[i],
-                                                 0);
+          if (type == TYPE_CRT)
+            ret =
+              gnutls_x509_crt_set_key_purpose_oid (crt, cfg.key_purpose_oids[i], 0);
+          else
+            ret =
+              gnutls_x509_crq_set_key_purpose_oid (crt, cfg.key_purpose_oids[i], 0);
 
           if (ret < 0)
             {
@@ -848,7 +881,7 @@ get_days (void)
 
   if (batch)
     {
-      if (cfg.expiration_days <= 0)
+      if (cfg.expiration_days == 0 || cfg.expiration_days < -2)
         return 365;
       else
         return cfg.expiration_days;
@@ -874,7 +907,7 @@ get_ca_status (void)
   else
     {
       return
-        read_yesno ("Does the certificate belong to an authority? (y/N): ");
+        read_yesno ("Does the certificate belong to an authority? (y/N): ", 0);
     }
 }
 
@@ -889,7 +922,7 @@ get_crq_extensions_status (void)
     {
       return
         read_yesno
-        ("Do you want to honour the extensions from the request? (y/N): ");
+        ("Do you want to honour the extensions from the request? (y/N): ", 0);
     }
 }
 
@@ -951,7 +984,7 @@ get_tls_client_status (void)
     }
   else
     {
-      return read_yesno ("Is this a TLS web client certificate? (y/N): ");
+      return read_yesno ("Is this a TLS web client certificate? (y/N): ", 0);
     }
 }
 
@@ -965,7 +998,7 @@ get_tls_server_status (void)
   else
     {
       return
-        read_yesno ("Is this also a TLS web server certificate? (y/N): ");
+        read_yesno ("Is this a TLS web server certificate? (y/N): ", 0);
     }
 }
 
@@ -1357,11 +1390,11 @@ get_sign_status (int server)
     {
       if (server)
         msg =
-          "Will the certificate be used for signing (DHE and RSA-EXPORT ciphersuites)? (y/N): ";
+          "Will the certificate be used for signing (DHE and RSA-EXPORT ciphersuites)? (Y/n): ";
       else
         msg =
-          "Will the certificate be used for signing (required for TLS)? (y/N): ";
-      return read_yesno (msg);
+          "Will the certificate be used for signing (required for TLS)? (Y/n): ";
+      return read_yesno (msg, 1);
     }
 }
 
@@ -1378,11 +1411,11 @@ get_encrypt_status (int server)
     {
       if (server)
         msg =
-          "Will the certificate be used for encryption (RSA ciphersuites)? (y/N): ";
+          "Will the certificate be used for encryption (RSA ciphersuites)? (Y/n): ";
       else
         msg =
-          "Will the certificate be used for encryption (not required for TLS)? (y/N): ";
-      return read_yesno (msg);
+          "Will the certificate be used for encryption (not required for TLS)? (Y/n): ";
+      return read_yesno (msg, 1);
     }
 }
 
@@ -1397,7 +1430,7 @@ get_cert_sign_status (void)
     {
       return
         read_yesno
-        ("Will the certificate be used to sign other certificates? (y/N): ");
+        ("Will the certificate be used to sign other certificates? (y/N): ", 0);
     }
 }
 
@@ -1411,7 +1444,7 @@ get_crl_sign_status (void)
   else
     {
       return
-        read_yesno ("Will the certificate be used to sign CRLs? (y/N): ");
+        read_yesno ("Will the certificate be used to sign CRLs? (y/N): ", 0);
     }
 }
 
@@ -1425,7 +1458,7 @@ get_code_sign_status (void)
   else
     {
       return
-        read_yesno ("Will the certificate be used to sign code? (y/N): ");
+        read_yesno ("Will the certificate be used to sign code? (y/N): ", 0);
     }
 }
 
@@ -1440,7 +1473,7 @@ get_ocsp_sign_status (void)
     {
       return
         read_yesno
-        ("Will the certificate be used to sign OCSP requests? (y/N): ");
+        ("Will the certificate be used to sign OCSP requests? (y/N): ", 0);
     }
 }
 
@@ -1455,7 +1488,7 @@ get_time_stamp_status (void)
     {
       return
         read_yesno
-        ("Will the certificate be used for time stamping? (y/N): ");
+        ("Will the certificate be used for time stamping? (y/N): ", 0);
     }
 }
 
@@ -1470,7 +1503,7 @@ get_ipsec_ike_status (void)
     {
       return
         read_yesno
-        ("Will the certificate be used for IPsec IKE operations? (y/N): ");
+        ("Will the certificate be used for IPsec IKE operations? (y/N): ", 0);
     }
 }
 
@@ -1668,6 +1701,26 @@ get_locality_crq_set (gnutls_x509_crq_t crq)
 
 }
 
+void
+get_dn_crq_set (gnutls_x509_crq_t crq)
+{
+  int ret;
+  const char* err;
+
+  if (batch)
+    {
+      if (!cfg.dn)
+        return;
+      ret =
+        gnutls_x509_crq_set_dn (crq, cfg.dn, &err);
+      if (ret < 0)
+        {
+          fprintf (stderr, "set_dn: %s at: %s\n", gnutls_strerror (ret), err);
+          exit (1);
+        }
+    }
+}
+
 void
 get_cn_crq_set (gnutls_x509_crq_t crq)
 {
